package gopher

import (
	"gophi/core"

	"github.com/grufwub/go-errors"
)

// Gopher specific error codes
var (
	InvalidGophermapErr  = errors.NewBaseError(invalidGophermapErrStr)
	SubgophermapIsDirErr = errors.NewBaseError(subgophermapIsDirErrStr)
	SubgophermapSizeErr  = errors.NewBaseError(subgophermapSizeErrStr)
)

// generateErrorResponse takes an error code and generates an error response byte slice
func generateErrorResponse(err *errors.Error) ([]byte, bool) {
	switch {
	case err.EqualsBase(core.ConnWriteErr):
		return nil, false // no point responding if we couldn't write
	case err.EqualsBase(core.ConnReadErr):
		return buildErrorLine(errorResponse503), true
	case err.EqualsBase(core.ConnCloseErr):
		return nil, false // no point responding if we couldn't close
	case err.EqualsBase(core.ListenerResolveErr):
		return nil, false // not user facing
	case err.EqualsBase(core.ListenerBeginErr):
		return nil, false // not user facing
	case err.EqualsBase(core.ListenerAcceptErr):
		return nil, false // not user facing
	case err.EqualsBase(core.InvalidIPErr):
		return nil, false // not user facing
	case err.EqualsBase(core.InvalidPortErr):
		return nil, false // not user facing
	case err.EqualsBase(core.MutexUpgradeErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(core.MutexDowngradeErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(core.FileOpenErr):
		return buildErrorLine(errorResponse404), true
	case err.EqualsBase(core.FileStatErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(core.FileReadErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(core.FileTypeErr):
		return buildErrorLine(errorResponse404), true
	case err.EqualsBase(core.DirectoryReadErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(core.RestrictedPathErr):
		return buildErrorLine(errorResponse403), true
	case err.EqualsBase(core.InvalidRequestErr):
		return buildErrorLine(errorResponse400), true
	case err.EqualsBase(core.CGIStartErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(core.CGIExitCodeErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(InvalidGophermapErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(SubgophermapIsDirErr):
		return buildErrorLine(errorResponse500), true
	case err.EqualsBase(SubgophermapSizeErr):
		return buildErrorLine(errorResponse500), true
	default:
		return nil, false
	}
}
