package gopher

import (
	"gophi/core"
	"os"

	"github.com/grufwub/go-errors"
)

// gophermapContents is an implementation of core.FileContents that holds individually renderable sections of a gophermap
type gophermapContent struct {
	sections []gophermapSection
}

// Load takes an open FD and loads the gophermap contents into memory as different renderable sections
func (gc *gophermapContent) Load(path *core.Path, file *os.File) *errors.Error {
	var err *errors.Error
	gc.sections, err = readGophermap(file, path)
	return err
}

// WriteToClient renders each cached section of the gophermap, and writes them to the client
func (gc *gophermapContent) WriteToClient(client *core.Client, path *core.Path) *errors.Error {
	// Render + write the sections!
	for _, section := range gc.sections {
		err := section.RenderAndWrite(client)
		if err != nil {
			return err
		}
	}

	// Finally, write the footer (including last-line)
	return client.Conn().Write(footer)
}

// Clear empties currently cached GophermapContents memory
func (gc *gophermapContent) Clear() {
	gc.sections = nil
}
